function [ramp,eventupall,eventdnall] = Ramp_FixedTime_Test(Power, Window, Thresh, dayfr, forecast, itwr, RampFlag, Aggflag, path2)
% find Ramps with Fixed Time method

ramp  = NaN(1,5,2);
timestep = (dayfr(2)-dayfr(1))*60*24;
windowpt = fix(Window/round(timestep)) + 1;
npoint = size(Power,1);
for imodel = 1:2
    power = Power(:,imodel);
    powererrup = NaN(size(power));
    powererrdn = NaN(size(power));
    for ipoint = 1:npoint - windowpt + 1
        krit = power(ipoint + windowpt - 1) - power(ipoint);
        if krit >= Thresh 
            powererrup(ipoint:ipoint + windowpt - 1) = 1;
        end
        if krit <= -Thresh 
            powererrdn(ipoint:ipoint + windowpt - 1) = -1;
        end
    end
    
    indexup = [];
    indexdn = [];
    eventup = 0;
    eventdn = 0;
    
    if powererrup(1)==1
        eventup = 1;
        indexup(1,1) = 1;
    end
    if powererrdn(1)==-1
        eventdn = 1;
        indexdn(1,1) = 1;
    end
    
    for i = 2:npoint
        if ~(powererrup(i-1) == 1) && powererrup(i) == 1
            eventup = eventup + 1;
            indexup(eventup,1) = i;
        end
        if powererrup(i-1) == 1 && ~(powererrup(i) == 1)
            indexup(eventup,2) = i-1;
        end
        if ~(powererrdn(i-1) == -1) && powererrdn(i) == -1
            eventdn = eventdn + 1;
            indexdn(eventdn,1) = i;
        end
        if powererrdn(i-1) == -1 && ~(powererrdn(i) == -1)
            indexdn(eventdn,2) = i-1;
        end
    end
    if powererrup(npoint) == 1
        indexup(end,2) = npoint;
    end
    if powererrdn(npoint) == -1
        indexdn(end,2) = npoint;
    end
    for ievent = 1:eventup
        deltaT = (indexup(ievent,2) - indexup(ievent,1))*timestep;
        deltaP = power(indexup(ievent,2)) - power(indexup(ievent,1));
        indexP = (indexup(ievent,2) + indexup(ievent,1))/2;
        ramp(ievent,:,imodel) = [indexP*timestep deltaT deltaP indexup(ievent,:)];
    end
    for ievent = 1:eventdn
        deltaT = (indexdn(ievent,2) - indexdn(ievent,1))*timestep;
        deltaP = power(indexdn(ievent,2)) - power(indexdn(ievent,1));
        indexP = (indexdn(ievent,2) + indexdn(ievent,1))/2;
        ramp(eventup + ievent,:,imodel) = [indexP*timestep deltaT deltaP indexdn(ievent,:)];
    end
    powerupall(:,imodel) = powererrup;
    powerdnall(:,imodel) = powererrdn;
    eventupall(imodel) = eventup;
    eventdnall(imodel) = eventdn;
end

if Aggflag
  twrttl = 'twr_{aggr}';
  twrstr = 'twr_aggr';
else
  twrttl = 'twr#1';
  twrstr = 'twr1';
end

if RampFlag && Window ==120 && Thresh == 0.4 && forecast==0 && itwr==1
    figure('name','Fixed Time Interval Method','position',[10 10 1950 950]);
    subplot(2,1,1);
    plot(dayfr, Power(:,1), 'k','linewidth',2);
    hold on
    plot(dayfr, Power(:,1), 'sk')
    plot(dayfr(~isnan(powerupall(:,1))),Power(~isnan(powerupall(:,1)),1),'sr')
    plot(dayfr(~isnan(powerdnall(:,1))),Power(~isnan(powerdnall(:,1)),1),'sg')
    plot(dayfr(~isnan(powerdnall(:,1)) & ~isnan(powerupall(:,1))),Power(~isnan(powerdnall(:,1)) & ~isnan(powerupall(:,1)),1),'sc','MarkerSize',12)
    set(gca,'LineWidth',2, 'FontWeight','bold','FontSize',10)
    ylabel({'\fontsize{10pt}\bf OBS';'power'})
    title({['\fontsize{16pt}\rm ' twrttl '   \fontsize{14pt} Frct' sprintf('%02g',forecast) '   \fontsize{16pt}  RAMP:' num2str(Thresh*100) '% over ' num2str(Window) ' minutes  \fontsize{14pt} windowwidth=' num2str(windowpt)]; '\fontsize{10pt}\rm Fixed Time Interval Method'})
    datetick(gca,'x','ddmmm','keepticks')
    axes('Position',[0.9 0.7 0.2 0.2],'visible','off');
    text(0.1,0.95,['\fontsize{14pt}\bf' num2str(eventupall(1))],'color','r')
    text(0.1,0.75,['\fontsize{14pt}\bf' num2str(eventdnall(1))],'color','g')
    grid on

    subplot(2,1,2)
    plot(dayfr, Power(:,2), 'k','linewidth',2)
    hold on
    plot(dayfr, Power(:,2), 'sk')
    plot(dayfr(~isnan(powerupall(:,2))),Power(~isnan(powerupall(:,2)),2),'sr')
    plot(dayfr(~isnan(powerdnall(:,2))),Power(~isnan(powerdnall(:,2)),2),'sg')
    plot(dayfr(~isnan(powerdnall(:,2)) & ~isnan(powerupall(:,2))),Power(~isnan(powerdnall(:,2)) & ~isnan(powerupall(:,2)),2),'sc','MarkerSize',12)
    set(gca,'LineWidth',2, 'FontWeight','bold','FontSize',10)
    ylabel({'\fontsize{10pt}\bf Model';'power'})
    datetick(gca,'x','ddmmm','keepticks')
    axes('Position',[0.9 0.4 0.2 0.2],'visible','off');
    text(0.1,0.2,['\fontsize{14pt}\bf' num2str(eventupall(2))],'color','r')
    text(0.1,0.05,['\fontsize{14pt}\bf' num2str(eventdnall(2))],'color','g')
    grid on 

    saveas(gcf,[path2 '\Timeseries_rampfixedtime_' num2str(Thresh*100) 'in' num2str(Window) '_' twrstr '_frct' sprintf('%02g',forecast) '.jpg'])
    close gcf
end