 function [WindObs2,WindMod2,ntower] = reading_windfiles2(ntower,path1,ncflag)
% This program reads the Observation and Model data from text or netcdf files,
% interpolates model data to observations time step, removes towers with
% less than 50% data in observations from analysis
% it should be modified according to user's data files format
warning off

badflag = -999;

% Read Observation Data file
if ncflag
    x = rd_netcdf([path1 'observations.nc']);
    WindObs(:,1) = x.time;
    x = rmfield(x,'time');
    fn = fieldnames(x);
    for i = 1:length(fn)
       WindObs(:,i+1) = x.(fn{i});
    end
else
    WindObs = Read_All_Towers_ObsData([path1 'observations.txt'], ntower);
    obslen = size(WindObs,1);
    ymdobs = datevec(datenum([WindObs(:,1) ones(obslen,1) ones(obslen,1)]) + WindObs(:,2) - 1);
    indobs = datenum([ymdobs(:,1:3) WindObs(:,3:4) zeros(obslen,1)]);
    WindObs(:,2:4)=[];
    WindObs(:,1) = indobs;
end
WindObs(WindObs == badflag) = NaN;


% Read Model Data files
if ncflag
    files = dir([path1 '*.nc']);
    k = 1;
    for i = 1:length(files)
        if isempty(findstr('observations',files(i).name ))
            x = rd_netcdf([path1 files(i).name]);
            WindFrct(:,1,k) = x.time;
            x = rmfield(x,'time');
            fn = fieldnames(x);
            for j = 1:length(fn)
              WindFrct(:,j+1,k) = x.(fn{i});  
            end
            k = k+1;
        end
    end
    ntower = size(WindFrct,2) - 1;
else
    XFrct = Read_All_Towers_FrctData(path1, ntower);
    nfiles = size(XFrct,3);
    modlen = size(XFrct,1);
    for ifile = 1:nfiles
        ymdmod = datevec(datenum([XFrct(:,1,ifile) ones(modlen,1) ones(modlen,1)]) + XFrct(:,2,ifile) - 1);
        indmod = datenum([ymdmod(:,1:3) XFrct(:,3:4,ifile) zeros(modlen,1)]);
        WindFrct(:,:,ifile) = [indmod XFrct(:,5:end,ifile)];
    end
end
WindFrct(WindFrct == badflag) = NaN;

ind = [];
for itwr = 1:ntower
    if sum(isnan(WindObs(:,1+itwr))) > 0.5*length(WindObs(:,1+itwr))
        ind = [ind; itwr];
    end
end
WindObs(:,1+ind) = [];
WindFrct(:,1+ind,:) = [];
ntower = ntower - length(ind);


for ifile = 1:nfiles
    indfrct = WindFrct(:,1,ifile);
    WindObs2(:,:,ifile) = WindObs(indobs >= indfrct(1) & indobs <= indfrct(end),:);
    indobs2 = WindObs2(:,1,ifile);
    WindMod2(:,1,ifile)=indobs2;
    for itwr = 1:ntower
      % interpolation of model data to observation timestep
      WindMod2(:,itwr+1,ifile)= interp1q(indfrct,WindFrct(:,1+itwr,ifile),indobs2);   
    end
end

