% Main program of the tool: 
% find Ramps for Forecast Run method data, calculate the scores for three methods
% and all Ramp definitions, produce a txt file with results, a final
% results plot and additional plots if that option was selected

% optional variables

% bonusweight = 0;
% WeightFlag = true;
% PowerFlag = false;
% 
% addplot = true;
% forecast = 3;
% method = 1;
% configuration = 1;
% aggregation = 0;
% nfrct <= frcthrs;

% Input data size: number of towers and forecasts
ntower = size(WindMod2,2)-1;
frcthrs = ceil((WindObs2(end,1,1) - WindObs2(1,1,1))*24);

%%% Apply the algorithm to aggregate of power or over all towers
nfiles = size(WindObs2,3);
if aggregation == 1
    for ifile=1:nfiles
        ObsData(:,:,ifile) = [WindObs2(:,1,ifile) nanavg(WindObs2(:,2:ntower+1,ifile))];
        ModData(:,:,ifile) = [WindMod2(:,1,ifile) nanavg(WindMod2(:,2:ntower+1,ifile))];
    end
    ntower=1;
else
  ObsData = WindObs2;
  ModData = WindMod2;
end

% Number of days in the input data and timestep
ndays = fix(ObsData(end,1,end) - ObsData(1,1,1)) + 1;
tstep = (ObsData(2,1,1)-ObsData(1,1,1))*24*60;

%%%%%%%%%%%%%%%%%%%%%%%%%% Ramp Definition %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%% Can be changed according to user's needs %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Window = [30,60,120,180];
Thresh = 0.7:-0.1:0.3;


% Size of Ramp Definition Matrix 
nwindow = length(Window);
nthresh = length(Thresh);
% Number of Ramp identification methods
nmethod = 3;

% ramp calculating functions
ramp_fun = {@(a,b,c,d,e,f,g,h) Ramp_FixedTime_Test(a,b,c,d,e,f,g,h);...
            @(a,b,c,d,e,f,g,h) Ramp_AdjTime_Test(a,b,c,d,e,f,g,h);...
            @(a,b,c,d,e,f,g,h) Ramp_Derivative_Test(a,b,c,d,e,f,g,h)};

% Weighting matrix definition: if option "weighted" is chosen, then it's a
% symmetric matrix, otherwise - matrix of ones
if WeightFlag 
    weighting(1,1) = 1;
    for i = 1:nthresh-1
      for j = 2:nwindow
          weighting(i,j) = weighting(i,j-1) - 0.1;
      end
      weighting(i+1,1) = weighting(i,1) - 0.1;
    end
    for j = 2:nwindow
        weighting(i+1,j) = weighting(i+1,j-1) - 0.1;
    end
          
    weighting = weighting./(mean(mean(weighting)));
else
    weighting = ones(nthresh,nwindow);
end

% Scores applied to events for each scenario
Scenario_Score = [1 0 -1 0.1*bonusweight 0  -1 0.1*bonusweight 1];
Min_Scenario_Score = [0.1*bonusweight NaN 0 NaN NaN 0.2*bonusweight   NaN 0.1*bonusweight];

% Initialization
Total_Score = zeros(nmethod,nfrct);
Total_Score_Up = zeros(nmethod,nfrct);
Total_Score_Dn = zeros(nmethod,nfrct);
Score = zeros(nthresh, nwindow, nmethod, nfrct);
Score_Up = zeros(nthresh, nwindow, nmethod, nfrct);
Score_Dn = zeros(nthresh, nwindow, nmethod, nfrct);
Count_Scenarios = zeros(8,nmethod,nfrct);
Count_Events = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Up = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Dn = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Obs = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Obs_Up = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Obs_Dn = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Mod = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Mod_Up = zeros(nthresh,nwindow,nmethod,nfrct);
Count_Events_Mod_Dn = zeros(nthresh,nwindow,nmethod,nfrct);
% ramp definition loop
for ithresh = 1:nthresh
  thresh = Thresh(ithresh);
  for iwindow = 1:nwindow
    window = Window(iwindow);
    DeltaTmin = [window,tstep,tstep]; 
    % forecast loop
    for ifiles = 1:nfiles
      % method loop
      for imet = 1:nmethod
         % tower loop
         for itwr = 1:ntower
             % if input data is windspeed, turn it into power
             if ~PowerFlag
                powerobs = speed2power(ObsData(:,itwr+1,ifiles))';
                powermod = speed2power(ModData(:,itwr+1,ifiles))';
             else
                powerobs = ObsData(:,itwr+1,ifiles);
                powermod = ModData(:,itwr+1,ifiles);
             end
             powermod(isnan(powerobs)) = NaN;
             powerobs(isnan(powermod)) = NaN;
             % identify ramps with current method in observation and model data 
            [Ramp, eventup, eventdn]  = ramp_fun{imet}([powerobs,powermod], window, thresh, ObsData(:,1,ifiles), 1, itwr, addplot, aggregation);
            data_Tower = Ramp(:,:,1);
            data_Model = Ramp(:,:,2);
            data_Tower(data_Tower(:,3)==0,:)=[];
            data_Model(data_Model(:,3)==0,:)=[];  
            if isempty(data_Tower)
                data_Tower = [NaN NaN NaN NaN NaN];
            end
            if isempty(data_Model)
                data_Model = [NaN NaN NaN NaN NaN];
            end
            % Count ramp events in observation and model data: up&down
            % events, up events and down events
            
            % number of events in observation and model data
            nobs = size(data_Tower,1);
            nmdl = size(data_Model,1);
            
            
            Time_Ramps_Tower = data_Tower(:,1);
            DeltaT_Ramps_Tower = data_Tower(:,2);
            DeltaP_Ramps_Tower = data_Tower(:,3);
            Ini_Time_Ramps_Tower = data_Tower(:,4)*tstep;
            Fin_Time_Ramps_Tower = data_Tower(:,5)*tstep;
            Event_Frct_Hour_Tower = fix(Time_Ramps_Tower/60)+1; 

            Time_Ramps_Model = data_Model(:,1);
            DeltaT_Ramps_Model = data_Model(:,2);
            DeltaP_Ramps_Model = data_Model(:,3);
            Ini_Time_Ramps_Model = data_Model(:,4)*tstep;
            Fin_Time_Ramps_Model = data_Model(:,5)*tstep;
            Event_Frct_Hour_Model = fix(Time_Ramps_Model/60)+1; 
            
            %%%%%%%%%%%%%%%%%%%%%%%%%% MODEL Matching %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            Matrix_Time_Shifts = NaN(nmdl,nobs);
            Matrix_Power_Shifts = NaN(nmdl,nobs);
            Matched_Points = NaN(nmdl,1);
            Delta_Time = NaN(nmdl,1);
            
            for ievent = 1:nmdl
                Matrix_Time_Shifts(ievent,:) = abs(Time_Ramps_Model(ievent) - Time_Ramps_Tower);
                Matrix_Power_Shifts(ievent,:) = abs(DeltaP_Ramps_Model(ievent)./DeltaT_Ramps_Model(ievent) - DeltaP_Ramps_Tower./DeltaT_Ramps_Tower);
            end

            while ~isnan(min(min(Matrix_Time_Shifts)))
                [r,c] = find(Matrix_Time_Shifts == min(min(Matrix_Time_Shifts)));
                locn = find(Matrix_Time_Shifts == min(min(Matrix_Time_Shifts)));
                if length(locn) == 1
                    Matched_Points(r) = c;
                    Delta_Time(r) = Matrix_Time_Shifts(r,c);
                    Matrix_Time_Shifts(r,:) = NaN;
                    Matrix_Time_Shifts(:,c) = NaN;
                    Matrix_Power_Shifts(r,:) = NaN;
                    Matrix_Power_Shifts(:,c) = NaN;
                else
                    whichminDeltaP = find(Matrix_Power_Shifts(locn) == min(Matrix_Power_Shifts(locn)));
                    if (length(whichminDeltaP) > 1)
                        whichminDeltaP = whichminDeltaP(1);
                    end
                    Matched_Points(r(whichminDeltaP)) = c(whichminDeltaP);
                    Delta_Time(r(whichminDeltaP)) = Matrix_Time_Shifts(r(whichminDeltaP),c(whichminDeltaP));
                    Matrix_Time_Shifts(r(whichminDeltaP),:) = NaN;
                    Matrix_Time_Shifts(:,c(whichminDeltaP)) = NaN;
                    Matrix_Power_Shifts(r(whichminDeltaP),:) = NaN;
                    Matrix_Power_Shifts(:,c(whichminDeltaP)) = NaN;
                end
            end

            Matched_Points(Delta_Time > window) = NaN;                

            %%%%%%%%%%%%%%%%%%%%%%%%% Model Score %%%%%%%%%%%%%%%%%%%%

            for i = 1:nmdl

            %%%%%%%%%%%%%%%%%%%%%%% Model UP RAMP %%%%%%%%%%%%%%%%%%%%%%%%%

             if (DeltaP_Ramps_Model(i) > 0)                                                        % UP RAMP event found in Model.

                if (isnan(Matched_Points(i)))                                                      % UP/NULL. No observed events fall in the Max_Timing_Error window. We are in SCENARIO 2.
                    if Event_Frct_Hour_Model(i) < nfrct+1
                        Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(2);
                        Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(2);
                        Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Count_Scenarios(2,imet,Event_Frct_Hour_Model(i)) = Count_Scenarios(2,imet,Event_Frct_Hour_Model(i))+1;

                        Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                    end
                elseif (DeltaP_Ramps_Tower(Matched_Points(i)) > 0)                                 % UP/UP. We are in SCENARIO 1. We might apply the bonus.

                    if (Time_Ramps_Model(i) > Time_Ramps_Tower(Matched_Points(i)) &&...            % Central time of the Model is late compared to the one observed, and
                        DeltaP_Ramps_Model(i) <= DeltaP_Ramps_Tower(Matched_Points(i)) && ...      % Model is underpredicting Delta power, and
                        Ini_Time_Ramps_Model(i) < Fin_Time_Ramps_Tower(Matched_Points(i)))         % Initial time of the forecast happens before the end time of the one observed.

                        BTau_scenario_1 = (abs(Delta_Time(i))/window)^bonusweight;                                               % Bonus Tau 1.
                        BAlpha_scenario_1 = (abs(DeltaP_Ramps_Model(i) - DeltaP_Ramps_Tower(Matched_Points(i))))^bonusweight;    % Bonus Alpha 1.

                    else                                                                                     % NO bonus applied.

                        BTau_scenario_1 = 1;                                                                 % NO Bonus Tau 1.
                        BAlpha_scenario_1 = 1;                                                               % NO Bonus Alpha 1.

                    end

                    Tau_scenario_1 = 1-(abs(Delta_Time(i))/window)*BTau_scenario_1;
                    Alpha_scenario_1 = 1-(abs(DeltaP_Ramps_Model(i) - DeltaP_Ramps_Tower(Matched_Points(i))))*BAlpha_scenario_1; 
                    Lambda_scenario_1 = (1-abs(DeltaT_Ramps_Model(i) - DeltaT_Ramps_Tower(Matched_Points(i)))/...
                                              (DeltaT_Ramps_Model(i) + DeltaT_Ramps_Tower(Matched_Points(i))));
                    
                    if Event_Frct_Hour_Model(i) < nfrct+1
                        Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(1)*(nthroot(Alpha_scenario_1*Tau_scenario_1*Lambda_scenario_1,3))+...
                                          Min_Scenario_Score(1)*(1-nthroot(Alpha_scenario_1*Tau_scenario_1*Lambda_scenario_1,3));
                        Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(1)*(nthroot(Alpha_scenario_1*Tau_scenario_1*Lambda_scenario_1,3))+...
                                          Min_Scenario_Score(1)*(1-nthroot(Alpha_scenario_1*Tau_scenario_1*Lambda_scenario_1,3));              

                        Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;

                        Count_Scenarios(1,imet,Event_Frct_Hour_Model(i)) = Count_Scenarios(1,imet,Event_Frct_Hour_Model(i))+1;

                        Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Obs_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                    end 

                elseif (DeltaP_Ramps_Tower(Matched_Points(i)) < 0)                                 % UP/DOWN. We are in SCENARIO 3. We DO NOT apply bonus.

                    BTau_scenario_3 = 1;                                                                     % NO Bonus Tau 3.
                    BAlpha_scenario_3 = 1;                                                                   % NO Bonus Alpha 3.

                    Tau_scenario_3 = (1-abs(Delta_Time(i))/window)*BTau_scenario_3;
                    Alpha_scenario_3 = (abs(DeltaP_Ramps_Model(i) - DeltaP_Ramps_Tower(Matched_Points(i)))/2)*BAlpha_scenario_3;   
                    Lambda_scenario_3 = 2*DeltaTmin(imet)/(DeltaT_Ramps_Model(i) + DeltaT_Ramps_Tower(Matched_Points(i)));
                    if (Lambda_scenario_3 > 1)
                        Lambda_scenario_3 = 1;
                    end
                    
                    if Event_Frct_Hour_Model(i) < nfrct+1
                        Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(3)*(nthroot(Alpha_scenario_3*Tau_scenario_3*Lambda_scenario_3,3))+...
                                          Min_Scenario_Score(3)*(1-nthroot(Alpha_scenario_3*Tau_scenario_3*Lambda_scenario_3,3));
                        Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(3)*(nthroot(Alpha_scenario_3*Tau_scenario_3*Lambda_scenario_3,3))+...
                                          Min_Scenario_Score(3)*(1-nthroot(Alpha_scenario_3*Tau_scenario_3*Lambda_scenario_3,3));
                        Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Count_Scenarios(3,imet,Event_Frct_Hour_Model(i)) = Count_Scenarios(3,imet,Event_Frct_Hour_Model(i))+1;

                        Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Obs_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                    end
                end

            %%%%%%%%%%%%%%%%%%%%%%% Model DOWN RAMP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            elseif (DeltaP_Ramps_Model(i) < 0)                                                     % DOWN RAMP event found in Model.

                if (isnan(Matched_Points(i)))                                                      % DOWN/NULL, No observed events fall in the Max_Timing_Error window. We are in SCENARIO 7.
                    if Event_Frct_Hour_Model(i) < nfrct+1
                        Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(7);
                        Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + Scenario_Score(7);
                        Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Count_Scenarios(7,imet,Event_Frct_Hour_Model(i)) = Count_Scenarios(7,imet,Event_Frct_Hour_Model(i))+1;
                        Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                    end
                elseif (DeltaP_Ramps_Tower(Matched_Points(i)) > 0)                                 % DOWN/UP. We are in SCENARIO 6. We might apply the bonus.

                    if ((Time_Ramps_Model(i) < Time_Ramps_Tower(Matched_Points(i)) &&...           % (Central time of the Model is early compared to the one observed, and
                        Fin_Time_Ramps_Model(i) > Ini_Time_Ramps_Tower(Matched_Points(i))) || ...  % the final time of the forecast happens after the beginning time of the one observed), OR
                       (Time_Ramps_Model(i) > Time_Ramps_Tower(Matched_Points(i)) &&...            % (Central time of the Model is late compared to the one observed, and
                        Ini_Time_Ramps_Model(i) < Fin_Time_Ramps_Tower(Matched_Points(i))))        % the initial time of the forecast happens before the end time of the one observed)

                        BTau_scenario_6 = (abs(Delta_Time(i))/window)^bonusweight;                 % Bonus Tau 6.
                        BAlpha_scenario_6 = (abs(DeltaP_Ramps_Model(i) - DeltaP_Ramps_Tower(Matched_Points(i)))/2)^bonusweight; % Bonus Alpha 6.

                    else

                        BTau_scenario_6 = 1;                                                                 % NO Bonus Tau 6.
                        BAlpha_scenario_6 = 1;                                                               % NO Bonus Alpha 6.

                    end

                    Tau_scenario_6 = 1-(abs(Delta_Time(i))/window)*BTau_scenario_6;
                    Alpha_scenario_6 = (abs(DeltaP_Ramps_Model(i) - DeltaP_Ramps_Tower(Matched_Points(i)))/2)*BAlpha_scenario_6;
                    Lambda_scenario_6 = 2*DeltaTmin(imet)/(DeltaT_Ramps_Model(i) + DeltaT_Ramps_Tower(Matched_Points(i)));   
                    if (Lambda_scenario_6 > 1)
                        Lambda_scenario_6 = 1;
                    end
                    
                    if Event_Frct_Hour_Model(i) < nfrct+1
                        Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) +  Scenario_Score(6)*(nthroot(Alpha_scenario_6*Tau_scenario_6*Lambda_scenario_6,3))+...
                                     Min_Scenario_Score(6)*(1-nthroot(Alpha_scenario_6*Tau_scenario_6*Lambda_scenario_6,3));
                        Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) +  Scenario_Score(6)*(nthroot(Alpha_scenario_6*Tau_scenario_6*Lambda_scenario_6,3))+...
                                     Min_Scenario_Score(6)*(1-nthroot(Alpha_scenario_6*Tau_scenario_6*Lambda_scenario_6,3));
                        Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;

                        Count_Scenarios(6,imet,Event_Frct_Hour_Model(i)) = Count_Scenarios(6,imet,Event_Frct_Hour_Model(i)) + 1;

                        Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Obs_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs_Up(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                    end

                elseif (DeltaP_Ramps_Tower(Matched_Points(i)) < 0)                                 % DOWN/DOWN. We are in SCENARIO 8. We might apply the bonus.


                    if (Time_Ramps_Model(i) < Time_Ramps_Tower(Matched_Points(i)) &&...            % Central time of the Model is ealry compared to the one observed, and
                        DeltaP_Ramps_Model(i) <= DeltaP_Ramps_Tower(Matched_Points(i)) && ...      % Model is underpredicting Delta power, and
                        Fin_Time_Ramps_Model(i) > Ini_Time_Ramps_Tower(Matched_Points(i)))         % Final time of the forecast happens alfter the initial time of the one observed.

                        BTau_scenario_8 = (abs(Delta_Time(i))/window)^bonusweight;                 % Bonus Tau 8.
                        BAlpha_scenario_8 = (abs(DeltaP_Ramps_Model(i) - DeltaP_Ramps_Tower(Matched_Points(i))))^bonusweight; % Bonus Alpha 8.

                    else                                                                                     % NO bonus applied.

                        BTau_scenario_8 = 1;                                                                 % NO Bonus Tau 1.
                        BAlpha_scenario_8 = 1;                                                               % NO Bonus Alpha 1.

                    end

                    Tau_scenario_8 = 1-(abs(Delta_Time(i))/window)*BTau_scenario_8;
                    Alpha_scenario_8 = 1-(abs(DeltaP_Ramps_Model(i) - DeltaP_Ramps_Tower(Matched_Points(i))))*BAlpha_scenario_8; 
                    Lambda_scenario_8 = (1-abs(DeltaT_Ramps_Model(i) - DeltaT_Ramps_Tower(Matched_Points(i)))/...
                                              (DeltaT_Ramps_Model(i) + DeltaT_Ramps_Tower(Matched_Points(i))));
                    
                    if Event_Frct_Hour_Model(i) < nfrct+1
                        Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) +  Scenario_Score(8)*(nthroot(Alpha_scenario_8*Tau_scenario_8*Lambda_scenario_8,3))+...
                                     Min_Scenario_Score(8)*(1-nthroot(Alpha_scenario_8*Tau_scenario_8*Lambda_scenario_8,3));
                        Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;
                        Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) +  Scenario_Score(8)*(nthroot(Alpha_scenario_8*Tau_scenario_8*Lambda_scenario_8,3))+...
                                     Min_Scenario_Score(8)*(1-nthroot(Alpha_scenario_8*Tau_scenario_8*Lambda_scenario_8,3));
                        Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) = Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Model(i)) + 1;

                        Count_Scenarios(8,imet,Event_Frct_Hour_Model(i)) = Count_Scenarios(8,imet,Event_Frct_Hour_Model(i))+1;

                        Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Obs_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Obs_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                        Count_Events_Mod_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) = Count_Events_Mod_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Model(i)) + 1;
                    end
                end

              end

            end

            Event_Frct_Hour_Tower(Matched_Points(~isnan(Matched_Points))) = [];
            DeltaP_Ramps_Tower(Matched_Points(~isnan(Matched_Points))) = [];            % We remove the events in the Tower that have been matched.

            for i = 1:length(DeltaP_Ramps_Tower)

                %%%%%%%%%%%%%%%%%%%%%%% Tower UP RAMP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

               if (DeltaP_Ramps_Tower(i) > 0)                                                                   % NULL/UP. UP RAMP event found in Tower. It was not previously matched to any model ramp. We are in SCENARIO 4.
                 if Event_Frct_Hour_Tower(i) < nfrct+1
                     Score(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + Scenario_Score(4);
                     Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + 1;
                     Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Score_Up(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + Scenario_Score(4);
                     Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Count_Events_Up(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + 1;

                     Count_Scenarios(4,imet,Event_Frct_Hour_Tower(i)) = Count_Scenarios(4,imet,Event_Frct_Hour_Tower(i))+1;

                     Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) = Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) + 1;
                     Count_Events_Obs_Up(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) = Count_Events_Obs_Up(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) + 1;
                 end
               %%%%%%%%%%%%%%%%%%%%%%% Tower DOWN RAMP %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
               elseif (DeltaP_Ramps_Tower(i) < 0)                                                               % NULL/DOWN. DOWN RAMP event found in Tower. It was not previously matched to any model ramp. We are in SCENARIO 5.
                if Event_Frct_Hour_Tower(i) < nfrct+1
                 Score(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Score(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + Scenario_Score(5);
                 Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Count_Events(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + 1;
                 Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Score_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + Scenario_Score(5);
                 Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) = Count_Events_Dn(ithresh,iwindow,imet,Event_Frct_Hour_Tower(i)) + 1;
                 
                 Count_Scenarios(5,imet,Event_Frct_Hour_Tower(i)) = Count_Scenarios(5,imet,Event_Frct_Hour_Tower(i)) + 1;
                 
                 Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) = Count_Events_Obs(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) + 1;
                 Count_Events_Obs_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) = Count_Events_Obs_Dn(ithresh, iwindow, imet, Event_Frct_Hour_Tower(i)) + 1;
                end 
               end       

            end

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         end
      end
    end
  end
end

%% Total score as average over all Ramp Definitions
for ifrct = 1:nfrct
    for imet = 1:nmethod
        Total_Score(imet,ifrct)=sum(sum(Score(:,:,imet,ifrct).*weighting))/sum(sum(Count_Events(:,:,imet,ifrct)));
        Total_Score_Up(imet,ifrct)=sum(sum(Score_Up(:,:,imet,ifrct).*weighting))/sum(sum(Count_Events_Up(:,:,imet,ifrct)));
        Total_Score_Dn(imet,ifrct)=sum(sum(Score_Dn(:,:,imet,ifrct).*weighting))/sum(sum(Count_Events_Dn(:,:,imet,ifrct)));
    end
end
Total_Score(isnan(Total_Score))=0;
Total_Score_Up(isnan(Total_Score_Up))=0;
Total_Score_Dn(isnan(Total_Score_Dn))=0;
%%%%%%%%%%%%%%%%%%% Plotting the results %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% Construct plot captions and filenames %%%%%%%%%%%%%%%%%%%%%
switch configuration
        case 1
            confstring = 'UpDownRamps';
            confstringplot = 'Up&Down Ramps';
        case 2
            confstring = 'UpRamps';
            confstringplot = 'Up Ramps';
        case 3
            confstring = 'DownRamps';
            confstringplot = 'Down Ramps';
end
switch aggregation
        case 1
            agrstring = '_aggr';
            agrstringplot = 'aggregate';
        case 0
            agrstring = '';
            agrstringplot = '';
end
        
switch method
        case 1
            methstring = 'FixedTime';
        case 2
            methstring = 'Min-Max';
        case 3
            methstring = 'Derivative';
end
switch WeightFlag
    case 1
        weightstring = 'weighted';
    case 0
        weightstring = 'unweighted';
end

frct = 0:nfrct-1;

delT = max(Window)/60/2;
x = [0 delT delT 0];
x1 = [frcthrs-1-delT nfrct-1 nfrct-1 frcthrs-1-delT];
allmin = min([min(Total_Score),min(Total_Score_Up),min(Total_Score_Dn)]);
allmax = max([max(Total_Score),max(Total_Score_Up),max(Total_Score_Dn)]);
y = [allmin allmin allmax+0.05 allmax+0.05];

% Total Model Score figure for Up&Down Events combined and separately
figure('name','Model Skill','position',[10 10 1950 950])
subplot(2,1,1)
plot(frct,Total_Score(1,:),'r',frct,Total_Score(2,:),'g',frct,Total_Score(3,:),'b','Linewidth',2)
hold on
plot(frct,Total_Score(1,:),'*r',frct,Total_Score(2,:),'*g',frct,Total_Score(3,:),'*b','Linewidth',2)
patch(x,y,'b','FaceAlpha',0.25);
if nfrct-1 > frcthrs-1 - delT
    patch(x1,y,'b','FaceAlpha',0.25);
end
set(gca, 'FontWeight','bold','FontSize',10,'LineWidth',2,'XTick',frct,'Ylim',[allmin allmax+0.05])
ylabel('Score')
h_legend = legend('Fixed Time','Min-Max','Derivative');
set(h_legend,'FontSize',7,'FontWeight','bold');
grid on
title({'Forecast Run Method','Model Score Up&Down Events combined ',[weightstring ' bonus ' num2str(bonusweight) ' ' agrstringplot]} )

subplot(2,1,2)
plot(frct,Total_Score_Up(1,:),'--r',frct,Total_Score_Up(2,:),'--g',frct,Total_Score_Up(3,:),'--b','Linewidth',2)
hold on
plot(frct,Total_Score_Dn(1,:),'r',frct,Total_Score_Dn(2,:),'g',frct,Total_Score_Dn(3,:),'b','Linewidth',2)
plot(frct,Total_Score_Up(1,:),'dr',frct,Total_Score_Up(2,:),'dg',frct,Total_Score_Up(3,:),'db','Linewidth',2)
plot(frct,Total_Score_Dn(1,:),'or',frct,Total_Score_Dn(2,:),'og',frct,Total_Score_Dn(3,:),'ob','Linewidth',2)
patch(x,y,'b','FaceAlpha',0.25);
if nfrct-1 > frcthrs-1 - delT
    patch(x1,y,'b','FaceAlpha',0.25);
end
set(gca, 'FontWeight','bold','FontSize',10,'LineWidth',2,'XTick',frct,'Ylim',[allmin allmax+0.05])
xlabel({'Forecast Hour','CIRES/NOAA RAMP Tool'})
ylabel('Score')
h_legend = legend('Events UP Fixed Time','Events UP Min-Max','Events UP Derivative','Events DOWN Fixed Time','Events DOWN Min-Max','Events DOWN Derivative');
set(h_legend,'FontSize',7,'FontWeight','bold');
grid on
title({'Model Score Up Events & Down Events separately',[weightstring ' bonus ' num2str(bonusweight) ' ' agrstringplot]})

saveas(gcf,[path2 'Total_Score_' weightstring '_bonus' num2str(bonusweight) agrstring '.jpg'])

% If additional plots were requested by user
if addplot
    % Number of events in each scenario figure
    Scen_bar = Count_Scenarios(:,:,forecast)./(ntower*ndays);
    figure('name','Number of Events per each scenario','position',[10 10 1950 950]);
    bar_handle = bar(Scen_bar,'grouped');
    set(bar_handle(1),'FaceColor','r')
    set(bar_handle(3),'FaceColor','b')
    set(gca,'Linewidth',2,'Fontweight','bold')
    xlabel({'Scenario #','','CIRES/NOAA RAMP Tool'})
    ylabel('# of Events')
    legend('Fixed Time','Min-Max','Derivative','Location','NorthWest')
    title({'Forecast Run Method',['Number of ' confstringplot ' in each of scenarios per tower per day for all ramp definitions ' agrstringplot]})
   
    saveas(gcf,[path2 'Number_of_' confstring '_per_scenario_frct' num2str(forecast-1) agrstring '.jpg'])
    
    Events_Matrix_Obs = Count_Events_Obs(:,:,method,forecast)./(ntower*ndays);
    Events_Matrix_Mod = Count_Events_Mod(:,:,method,forecast)./(ntower*ndays);
    Skill_Matrix = Score(:,:,method,forecast)./Count_Events(:,:,method,forecast);
    if configuration == 2
        Events_Matrix_Obs = Count_Events_Obs_Up(:,:,method,forecast)./(ntower*ndays);
        Events_Matrix_Mod = Count_Events_Mod_Up(:,:,method,forecast)./(ntower*ndays);
        Skill_Matrix = Score_Up(:,:,method,forecast)./Count_Events_Up(:,:,method,forecast);
    elseif configuration == 3
        Events_Matrix_Obs = Count_Events_Obs_Dn(:,:,method,forecast)./(ntower*ndays);
        Events_Matrix_Mod = Count_Events_Mod_Dn(:,:,method,forecast)./(ntower*ndays);
        Skill_Matrix = Score_Dn(:,:,method,forecast)./Count_Events_Dn(:,:,method,forecast);
    end
    
    
    lim1 = min(min(min(Events_Matrix_Obs)),min(min(Events_Matrix_Mod)));
    lim2 = max(max(max(Events_Matrix_Obs)),max(max(Events_Matrix_Mod)));
    
    % Number of events in Observation and Model data figure
    figure('name','Number of Events Matrix','position',[10 10 1950 950]);
    h1 = subplot(1,2,1);
    h1pos = get(h1,'position');
    imagesc(Events_Matrix_Obs);
    set(gca,'XTickLabel',Window,'XTick',1:nwindow,'YTick',1:nthresh,'YTickLabel',Thresh*100,'Linewidth',2,'Fontweight','bold')
    caxis([lim1 lim2])
    xlabel('\Delta Time (min)')
    ylabel('\Delta Power (%)')
    title({'Forecast Run Method',['# of ' confstringplot ' per tower per day ' agrstringplot],['Observations ' methstring ' Method Forecast ' num2str(forecast-1)]})
    
    h2 = subplot(1,2,2);
    imagesc(Events_Matrix_Mod);
    set(gca,'XTickLabel',Window,'XTick',1:nwindow,'YTick',1:nthresh,'YTickLabel',Thresh*100,'Linewidth',2,'Fontweight','bold')
    caxis([lim1 lim2])
    h2pos = get(h2,'position');
    colorbar
    set(h2,'position',h2pos)
    xlabel('\Delta Time (min)')
    ylabel('\Delta Power (%)')
    title({'Forecast Run Method',['# of ' confstringplot ' per tower per day ' agrstringplot],['Model ' methstring ' Method Forecast ' num2str(forecast-1)]})
    
    h3 = axes;
    set(h3,'Visible','off')
    set(h3,'position', [h1pos(1:2) h2pos(1)+h2pos(3)-h1pos(1) h1pos(4)]);
    xlabel({' ','\bf CIRES/NOAA Ramp Tool'},'visible','on')

    saveas(gcf,[path2 'Number_of_' confstring '_matrix_frct' num2str(forecast-1) '_method_' methstring agrstring '.jpg'])
    
    % Skill matrix figure for chosen forecast and method
    figure('name','Skill Matrix');
    imagesc(Skill_Matrix)
    set(gca,'XTickLabel',Window,'XTick',1:nwindow,'YTick',1:nthresh,'YTickLabel',Thresh*100,'Linewidth',2,'Fontweight','bold')
    colorbar
    xlabel({'\Delta Time (min)','CIRES/NOAA RAMP Tool'})
    ylabel('\Delta Power (%)')
    title({'Forecast Run Method',['Model Score ' weightstring ' for bonus ' num2str(bonusweight) ' ' agrstringplot],[methstring ' Method  Forecast ' num2str(forecast-1)]})
     
    saveas(gcf,[path2 'ModelSkill_matrix_for_' confstring '_' weightstring '_bonus' num2str(bonusweight) '_frct' num2str(forecast-1) '_method_' methstring agrstring '.jpg'])    
end
% saving the output textfile
Total_Score = [Total_Score; Total_Score_Up; Total_Score_Dn];
save([path2 'Total_Score_' weightstring '_bonus' num2str(bonusweight) agrstring '.txt'],'Total_Score','-ascii', '-tabs')