function [WindObs, WindMod, ntower] = reading_windfiles(ntower,nfrct,path1,ncflag)
% This program reads the Observation and Model data from text or netcdf files,
% interpolates model data to observations time step, removes towers with
% less than 50% data in observations from analysis
% it should be modified according to user's data files format
warning off

badflag = -999;

% Read Observation Data file
if ncflag
    x = rd_netcdf([path1 'observations.nc']);
    WindObs(:,1) = x.time;
    x = rmfield(x,'time');
    fn = fieldnames(x);
    for i = 1:length(fn)
       WindObs(:,i+1) = x.(fn{i});
    end
else
    WindObs = Read_All_Towers_ObsData([path1 'observations.txt'], ntower);
    obslen = size(WindObs,1);
    ymdobs = datevec(datenum([WindObs(:,1) ones(obslen,1) ones(obslen,1)]) + WindObs(:,2) - 1);
    indobs = datenum([ymdobs(:,1:3) WindObs(:,3:4) zeros(obslen,1)]);
    WindObs(:,2:4)=[];
    WindObs(:,1) = indobs;
end
WindObs(WindObs == badflag) = NaN;


% Read Model Data file
if ncflag
    files = dir([path1 '*.nc']);
    k = 1;
    for i = 1:length(files)
        if isempty(findstr('observations',files(i).name ))
            x = rd_netcdf([path1 files(i).name]);
            WindCtrl(:,1,k) = x.time;
            x = rmfield(x,'time');
            fn = fieldnames(x);
            for j = 1:length(fn)
              WindCtrl(:,j+1,k) = x.(fn{i});  
            end
            k = k+1;
        end
    end
    ntower = size(WindCtrl,2) - 1;
else
    for ifrct = 1:nfrct 
      XCtrl = Read_All_Towers_ModelData(path1, ifrct-1, ntower);
      modlen = size(XCtrl,1);
      ymdmod = datevec(datenum([XCtrl(:,1) ones(modlen,1) ones(modlen,1)]) + XCtrl(:,2) - 1);
      indmod = datenum([ymdmod(:,1:3) XCtrl(:,3:4) zeros(modlen,1)]);
      WindCtrl(:,:,ifrct) = [indmod XCtrl(:,5:end)];
    end
    
end
WindCtrl(WindCtrl == badflag) = NaN;

ind = [];
for itwr = 1:ntower
    if sum(isnan(WindObs(:,1+itwr))) > 0.5*length(WindObs(:,1+itwr))
        ind = [ind; itwr];
    end
end
WindObs(:,1+ind) = [];
WindCtrl(:,1+ind,:) = [];
ntower = ntower - length(ind);
indobs = WindObs(:,1);
for ifrct = 1:nfrct
    % Turn model date and time into one dayfraction variable
    indmod = WindCtrl(:,1,ifrct);
    WindMod(:,1,ifrct)=indobs;
    for itwr = 1:ntower
      % interpolation of model data to observation timestep
      WindMod(:,itwr+1,ifrct)= interp1q(indmod,WindCtrl(:,1+itwr,ifrct),indobs);   
    end
end


